@@ -22,6 +22,7 @@ use crc32fast::Hasher;
 use tokio::io::{AsyncRead, AsyncReadExt, AsyncWrite, BufReader, ReadBuf, Take};
 
 /// An entry within a larger ZIP file reader.
+#[derive(Debug)]
 pub struct ZipEntry {
     pub(crate) name: String,
     pub(crate) comment: Option<String>,
@@ -109,13 +110,7 @@ pub struct ZipEntryReader<'a, R: AsyncRead + Unpin> {
 impl<'a, R: AsyncRead + Unpin> ZipEntryReader<'a, R> {
     /// Construct an entry reader from its raw parts (a shared reference to the entry and an inner reader).
     pub(crate) fn from_raw(entry: &'a ZipEntry, reader: CompressionReader<'a, R>, stream: bool) -> Self {
-        ZipEntryReader {
-            entry,
-            reader,
-            stream,
-            hasher: Hasher::new(),
-            consumed: false,
-        }
+        ZipEntryReader { entry, reader, stream, hasher: Hasher::new(), consumed: false }
     }
 
     /// Returns a reference to the inner entry's data.
